function this = init_groups( this, s )

% this = init_groups( this, settings )
%
% settings, valid fields:
%   .setup_mode        group setup mode.
%   .setup_params      parameter(s) for the chosen setup mode.
%   .Reinit_model      settings for the Reinit_model primitive operation.
%   .Improve_model     settings for the Improve_model primitive operation.
%
% setup modes:
%   'percentage'   'setup_params' percent of all variables are
%                  chosen at random and an own group is created for
%                  each. At least 2 variables are chosen always.
%   'explicit'     'setup_params' specifies the grouping. format:
%                  {Vars1, ..., VarsN}, where VarsX is a vector
%                  defining variables in the X:th group.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


% Adds the requested groups to the current State. Each new group's model
% will also be tuned.
%
% Variables are added with 'find_or_create' and the new group
% is tuned with primitive ops 'Reinit_model and 'Improve_model', in this
% order. If the State already contains some of the needed groups in the
% group cache, they will be used.
%
% Note: It is NOT allowed to try to add variables which are already active
% in the State! This will result in a corrupted State. This means that the
% 'percentage' mode is allowed only for a State with no active variables.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global STATS State;
STATS = measure( STATS, 'init_groups', 'start', State );

% choose the grouping
switch s.setup_mode
  
 case 'percentage'
  [dummy, varcount] = get_data_dimension( State.modeler_class, State.data );
  vars = randperm( varcount );
  if length(vars) < 2, error('Data should contain at least 2 variables!'); end
  vars = vars( 1 : max( 2, round((s.setup_params / 100) * length(vars)) ) );
  grouping = num2cell( vars ); clear vars;
  
 case 'explicit'
  grouping = s.setup_params;
  
 otherwise
  error([ 'Unkown setup mode: ' s.setup_mode ]);
end


% add variables and tune their models
for g=1:length(grouping)
  
  % add a new group
  ind = find_or_create( sort(grouping{g}) );
  set_group_active( ind, 1 );
  
  % reinit its model
  Reinit_model( ind, s.Reinit_model, 'init_groups' );
  
  % tune its model
  Improve_model( ind, s.Improve_model, 'init_groups' );
  
  %STATS = set_active( STATS, 'revert' );
  STATS = measure( STATS, 'init_groups', 'checkpoint', State );
  
end


STATS = measure( STATS, 'init_groups', 'stop', State );
return;
