function new_cost = estimate_add_vars( this, source, vars )

% new_cost = estimate_add_vars( this, source, vars )
%
% this       target model
% source     source model
% vars       variables to be moved
%
% new_cost   new estimated cost of this group
%
% Estimates the new cost of this model if variables 'vars' were moved from
% source model to this model.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global State;


% some variable name changes
D = State.data;
Vars_from = vars;
types_from = D.types(Vars_from);
clear vars;

Params_from  = source.vq.params;
Params_to    = this.vq.params;
Vars_to      = this.vars;
types_to = D.types(Vars_to);

if isempty(Params_from) | isempty(Params_to)
  error( 'Both models should be in precomputed state!' );
end

Vars_from1=Vars_from(types_from==1);
Vars_from2=Vars_from(types_from==2);
Vars_to1=Vars_to(types_to==1);
Vars_to2=Vars_to(types_to==2);

ncentroids       = size(Params_to.Mu_bar, 1); 
datalen          = size(D.data, 1);
dim_from1         = length(Vars_from1);
dim_to1           = length(Vars_to1);
dim_new1          = dim_from1 + dim_to1;
dim_from2         = length(Vars_from2);
dim_to2           = length(Vars_to2);
dim_new2          = dim_from2 + dim_to2;
dim_from         = length(Vars_from);
dim_to           = length(Vars_to);
dim_new          = dim_from + dim_to;
[Vars_new, IndsTemp] = sort([Vars_from Vars_to]);
[Vars_new1, Inds1Temp] = sort([Vars_from1 Vars_to1]);
[Vars_new2, Inds2Temp] = sort([Vars_from2 Vars_to2]);
[dummy, Inds]  = sort(IndsTemp);
[dummy, Inds1]  = sort(Inds1Temp);
[dummy, Inds2]  = sort(Inds2Temp);


%
% initialize Params, the new struct
%

Params.Mu_mu       = zeros(1, dim_new1);
Params.S2_mu       = zeros(1, dim_new1);
Params.Mu_bar      = zeros(ncentroids, dim_new1);
Params.Mu_tilde    = zeros(ncentroids, dim_new1);
Params.Alpha_ksi   = zeros(1, dim_new1);
Params.Beta_ksi    = zeros(1, dim_new1);
Params.Ksi_alpha   = zeros(ncentroids, dim_new1);
Params.Ksi_beta    = zeros(ncentroids, dim_new1);
Params.U_p         = zeros(1, dim_new2);
Params.U_hat       = {};

%
% copy the information that will remain unchanged to Params
%

% compute the indices of the variables to be moved

Vars1 = [];
Vars2 = [];
%for i = 1:length(Vars_from)
%  if sum(source.vars == Vars_from(i)), Vars = [Vars i]; end
%end
for i = 1:length(Vars_from1)
  if sum(source.vars == Vars_from1(i)), Vars1 = [Vars1 i]; end
end
for i = 1:length(Vars_from2)
  if sum(source.vars == Vars_from2(i)), Vars2 = [Vars2 i]; end
end

Inds_from1                   = Inds1(1:dim_from1);
Inds_from2                   = Inds2(1:dim_from2);
Inds_to1                     = Inds1(dim_from1+1:end);
Inds_to2                     = Inds2(dim_from2+1:end);

Params.Mu_mu(Inds_from1)     = Params_from.Mu_mu(Vars1);
Params.Mu_mu(Inds_to1)       = Params_to.Mu_mu;

Params.S2_mu(Inds_from1)     = Params_from.S2_mu(Vars1);
Params.S2_mu(Inds_to1)       = Params_to.S2_mu;

Params.Alpha_ksi(Inds_from1) = Params_from.Alpha_ksi(Vars1);
Params.Alpha_ksi(Inds_to1)   = Params_to.Alpha_ksi;

Params.Beta_ksi(Inds_from1)  = Params_from.Beta_ksi(Vars1);
Params.Beta_ksi(Inds_to1)    = Params_to.Beta_ksi;

Params.U_p(Inds_from2)       = Params_from.U_p(Vars2);
Params.U_p(Inds_to2)         = Params_to.U_p;

Params.C_prior              = Params_to.C_prior;
Params.logC                 = Params_to.logC;
Params.F                    = Params_to.F;
Params.W                    = Params_to.W;


% compute new values for Mu_bar, Mu_tilde, Ksi_alpha, Ksi_beta and U_hat
% for the moved variables

Mu_mu     = Params.Mu_mu(Inds_from1);
S2_mu     = Params.S2_mu(Inds_from1);
Alpha_ksi = Params.Alpha_ksi(Inds_from1);
Beta_ksi  = Params.Beta_ksi(Inds_from1);
Ksi_alpha = zeros(ncentroids, dim_from1);
Ksi_beta  = zeros(ncentroids, dim_from1);
F         = Params.F;
W         = Params.W;
Mu_bar    = zeros(ncentroids, dim_from1);
Temp1     = zeros(ncentroids, dim_from1);
S2_x      = zeros(ncentroids, dim_from1);
U_p       = Params.U_p;
U_hat     = {};

for j = 1:dim_from1
  Temp1(:,j)  = W * D.data(:,Vars_from1(j));
  Mu_bar(:,j) = Temp1(:,j)./F; 
  S2_x(:,j)   = sum(W .* ((repmat(D.data(:,Vars_from1(j))',ncentroids,1)- ...
			   repmat(Mu_bar(:,j),1,datalen)).^2 + ...
			  this.s.implicit_noisevar), 2)./F;
end
S2_x(S2_x < 1e-100) = 1e-100;
Temp2    = S2_x + repmat(S2_mu,ncentroids,1) .* repmat(F,1,dim_from1);
Mu_bar   = ((S2_x.*repmat(Mu_mu,ncentroids,1)) + (repmat(S2_mu,ncentroids,1).*Temp1))./Temp2;
Mu_tilde = (S2_x .* repmat(S2_mu,ncentroids,1))./Temp2;

Ksi_alpha = repmat(Alpha_ksi,ncentroids,1) + 0.5 * repmat(F,1,dim_from1);
for j = 1:dim_from1
  Ksi_beta(:,j)  = sum(W.*(repmat(Mu_tilde(:,j),1,datalen)+...
			   this.s.implicit_noisevar + ...
			   (repmat(Mu_bar(:,j),1,datalen)-...
			    repmat(D.data(:,Vars_from1(j))',ncentroids,1)).^2), 2);
end
Ksi_beta  = 0.5 * Ksi_beta + repmat(Beta_ksi,ncentroids,1);

for j=1:dim_from2
  for k=1:D.S(Vars_from2(j))
    U_hat{j}(:,k)=U_p(j) + W*(D.data(:,Vars_from2(j))==k);
  end
end

%
% insert the new information into Params
%

for j=1:dim_from2
  Params.U_hat{Inds_from2(j)} = U_hat{j};            
end                                                  
for j=1:dim_to2                                       
  Params.U_hat{Inds_to2(j)} = Params_to.U_hat{j};    
end

Params.Mu_bar(:, Inds_from1)    = Mu_bar;
Params.Mu_bar(:, Inds_to1)      = Params_to.Mu_bar;

Params.Mu_tilde(:, Inds_from1)  = Mu_tilde;
Params.Mu_tilde(:, Inds_to1)    = Params_to.Mu_tilde;

Params.Ksi_alpha(:, Inds_from1) = Ksi_alpha;
Params.Ksi_alpha(:, Inds_to1)   = Params_to.Ksi_alpha;

Params.Ksi_beta(:, Inds_from1)  = Ksi_beta;
Params.Ksi_beta(:, Inds_to1)    = Params_to.Ksi_beta;

% finally, compute the new cost
new_cost = vq_cost(D, Params, Vars_new, this.s.implicit_noisevar);

return;













