function varargout = mdl_vq_real_nominal( varargin )

% Constructors:
%
%   object = mdl_vq_real_nominal( vars, s )
%
%     vars   integer vector of variable numbers to be included in the model
%     s      settings, see below
%
%     The first form creates a new VQ model for the given variables. The model
%     is tuned only very briefly, so its cost will be a quite rough estimation.
%
%     The data is read from global variable State.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


% Limitations:
%   -The data should contain a small amount of random noise in the real 
%    variables (Before we add the implicit noise into some IVGA version.)
%    The modeler
%    will fail if it manages to create a perfect model.

% 'Static' methods:
%
%   mdl_vq_real_nominal( 'init', s )
%
%     s   settings, see below
%
%     Registers all internal recorded operations to the current global STATS
%     object. This should be called once for a new global STATS object before
%     any models are created. Recorded operations:
%       - 'mdl_vq_vq' (no extra vars) Calls to private method 'vq' are
%         recorded with this name.

% settings, valid fields:
%   .max_points         Maximum number of centroids
%   .ncentroids_def     Number of centroids in an 'empty' vq. (see new_vq.m)
%   .iters              Number of internal iterations on each reinit.
%   .check              How often garbage centroids are cleaned out on reinit.
%                       (vectquant_mex will get max(2,s.check), see
%                       vectquantM.m for details)
%   .implicit_noisevar  Variance of implicit noise
%
% $Id: mdl_vq_real_nominal.m,v 1.5 2006/04/12 18:20:44 pwagner Exp $
% $Name: ivga-1_14_3 $
                                                                                
% Internal data fields:
%
% class mdl_vq_real_nominal:
%   .vars
%   .s
%
%   .train
%     .ncentroids_init
%     .seed
%     .ninits
%     .iters
%     .improvements
%
%   .vq
%     .params
%     .cost
%     .ncentroids

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global STATS State;


% define the class so that matlab recognizes it when loading objects
if nargin == 0
  this.vars  = [];
  this.s     = [];
  this.train = [];
  this.vq    = [];
  varargout{1} = class( this, 'mdl_vq_real_nominal' );
  return;
end


% 'static' methods
if ischar( varargin{1} )

 if isarg( 'init' )
  % initialization, stats registration
   if ~( isstruct(State.data))
    error( 'the data object is not a struct' );
  end
  if ~( isnumeric(State.data.data) && ndims(State.data.data) == 2 )
    error( 'Not a suitable data matrix in the data object' );
  end
  if ~( size(State.data.data,2) == length(State.data.types ))
    error( 'data.types is not the same length as the rows of data.data' )
  end
  if ~( size(State.data.data,2) == length(State.data.S ))
    error( 'data.S is not the same length as the rows of data.data' )
  end   

  STATS = add_operation( STATS, 'mdl_vq_vq' );
  return;
 end
 
end

this.vars = varargin{1};
this.s    = varargin{2};

this.train = [];
this.vq.params     = [];
this.vq.cost       = inf;
this.vq.ncentroids = 0;

varargout{1} = class( this, 'mdl_vq_real_nominal' );

% generate a quick vq approximation
varargout{1} = reinit( varargout{1} );

return;
