function [success, details] = Merge( s )

% [success, details] = Merge( settings )
%
% IVGA primitive operation. Recorded properties:
%   cputimer, cputime, total cost
%
% settings, valid fields:
%   .Reinit_model    settings for Reinit_model, which is called for the new,
%                    combined group.
%   .Improve_model   settings for Improve_model, which is called for the new,
%                    combined group.
%
% Returns:
%   success   true if the attempted merge was good and kept, false otherwise.
%   details   currently not used.
%
% Tries to merge two random active groups. If the result is better, the new
% group is kept and the old ones are deactivated. The model of the group
% that would result from the merge will be reinited and tuned before the
% decision is made.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global STATS State;


% can't merge if only one group exists
if length(State.Ind) == 1, success = false; details = []; return; end


STATS = measure( STATS, 'Merge', 'start', State );
start_time = cputimer(STATS);

% retrieve the state transition limiter from the State
stl = State.state_transition_limiter;


% randomly choose two active groups
temp    = randperm( length(State.Ind) );
ind1    = State.Ind( temp(1) );
ind2    = State.Ind( temp(2) );

% is the plan accepted?
[stl, accept] = accept_move_plan( stl, State, 'Merge', [ind1, ind2] );
if accept
  
  % combine their variables
  Vars = sort([ State.Groups(ind1).Vars State.Groups(ind2).Vars ]);

  % fetch/create the new combined group and switch to it
  ind = find_or_create( Vars );
  set_group_active( ind1, 0, 'dont_purge' );
  set_group_active( ind2, 0, 'dont_purge' );
  set_group_active( ind,  1 );

  % improve the new group a bit and get all costs
  Reinit_model( ind, s.Reinit_model, 'Merge' );
  Improve_model( ind, s.Improve_model, 'Merge' );
  err_old = get_cost( State.Groups(ind1).Model ) + ...
            get_cost( State.Groups(ind2).Model );
  err_new = get_cost( State.Groups(ind).Model );

  % is the result accepted?
  [stl, accept] = accept_move_result( ...
      stl, State, cputimer(STATS) - start_time, err_new - err_old );
  if accept
    
    % result was accepted, keep this grouping and purge the old models
    State.Groups(ind1).Model = purge( State.Groups(ind1).Model );
    State.Groups(ind2).Model = purge( State.Groups(ind2).Model );
    
  else
    
    % result was rejected, switch back
    set_group_active( ind,  0 );
    set_group_active( ind1, 1 );
    set_group_active( ind2, 1 );
    
  end
  
end


% write return values
success = accept;
details = [];

% store the state transition limiter back to the State
State.state_transition_limiter = stl;


STATS = measure( STATS, 'Merge', 'checkpoint', State );
STATS = measure( STATS, 'Merge', 'stop', State );
return;
